"""
Checking specifics portal settings.
This package must run in a chosen package (needs app_config.py)
"""

import unittest2 as unittest

from base import IntegrationTestCase

from Products.CMFCore.utils import getToolByName

# adapt if any need to your testing utils module.
from collective.z3cform.chosen.tests.globals import *

class TestSetup(IntegrationTestCase):
    """Check Policy."""

    def test_Noop(self):
        self.assertEquals(True, True)

def test_suite():
    """."""
    suite = unittest.TestSuite()
    suite.addTests(
        unittest.defaultTestLoader.loadTestsFromName(
            __name__))
    return suite  




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__docformat__ = 'restructuredtext en'

"""
################################################################################


This file stores globals used in tests and doctests,
just import all variables from here and enjoy.


Lot of files generated by the collective.generic packages  will try to load user defined objects in user specific files.
The final goal is to regenerate easyly the test infrastructure on templates updates without impacting
user-specific test boilerplate.
We do not use paster local commands (insert/update) as it cannot determine witch is specific or not and we prefer to totally
separe generated stuff and what is user specific



If you need to edit something in this file, you must have better to do it in:


    - user_globals.py


################################################################################
"""

import ConfigParser
import os
import re
import sys
from copy import deepcopy
from pprint import pprint

from zope.component import adapts, getMultiAdapter, getAdapter, getAdapters
from zope import interface, schema
from zope.interface import implementedBy, providedBy
from zope.interface.verify import verifyObject
from zope.traversing.adapters import DefaultTraversable
import zope
from plone.testing import z2

from Acquisition import aq_inner, aq_parent, aq_self

from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from collective.z3cform.chosen.testing import print_contents, Browser


zope.component.provideAdapter(DefaultTraversable, [None])

FF2_USERAGENT =  'Mozilla/5.0 (Windows; U; Windows NT 5.1; fr; rv:1.8.1.14) Gecko/20080404 Firefox/2.0.0.14'
GENTOO_FF_UA = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/20090912 Gentoo Shiretoko/3.5.3'

cwd = os.path.dirname(__file__)

def get_interfaces(o):
    return [o for o in o.__provides__.interfaces()]

def errprint(msg):
    """Writes 'msg' to stderr and flushes the stream."""
    sys.stderr.write(msg)
    sys.stderr.flush()

def pstriplist(s):
    print '\n'.join([a.rstrip() for a in s.split('\n') if a.strip()])


class Request(zope.publisher.browser.TestRequest):
    def __setitem__(self, name, value):
        self._environ[name] = value
# alias
TestRequest = Request
def make_request(url='http://nohost/@@myview',form=None, *args,  **kwargs):
    r = Request(environ = {'SERVER_URL': url, 'ACTUAL_URL': url}, form=form, *args, **kwargs)
    zope.interface.alsoProvides(r, zope.annotation.interfaces.IAttributeAnnotatable)
    return r

# if you have plone.reload out there add an helper to use in doctests while programming
# just use preload(module) in pdb :)
# it would be neccessary for you to precise each module to reload, this method is also not recursive.
# eg: (pdb) from foo import bar;preload(bar)
def preload(modules_or_module, excludelist=None):
    try:
        modules = modules_or_module
        if not (isinstance(modules_or_module, list)
                or isinstance(modules_or_module, tuple)):
            modules = [modules_or_module]
        if not excludelist:
            excludelist = []
        import sys
        if not modules:
            modules = sys.modules
        from plone.reload.xreload import Reloader
        for module in modules:
            if not module in excludelist:
                try:
                    Reloader(module).reload()
                except Exception, e:
                    pass
    except Exception, e:
        print "Cant reload code: %s " % e

from collective.z3cform.chosen.testing import (
    login,
    logout,
    TEST_USER_NAME,
    TEST_USER_ROLES,
    TEST_USER_ID,
    SITE_OWNER_NAME,
    print_contents,
    PLONE_MANAGER_ID,
    PLONE_MANAGER_NAME,
    PLONE_MANAGER_PASSWORD,
)
# load user specific globals
try: from collective.z3cform.chosen.tests.user_globals import *
except: pass

# vim:set et sts=4 ts=4 tw=80:



from plone.z3cform import layout
from z3c.form import form, button, field
from z3c.formwidget.query.interfaces import IQuerySource
from zope.component import adapts
from zope.interface import Interface, implements
from zope import schema
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary

from ordereddict import OrderedDict

from collective.z3cform.chosen.widget import (
    AjaxChosenFieldWidget,
    AjaxChosenMultiFieldWidget,
    ChosenFieldWidget,
    ChosenMultiFieldWidget,
)

DATA = OrderedDict()
DATA["11"] ="aaa"
DATA["22"] ="bbb"
DATA["13"] ="ccc"
DATA["14"] ="ddd"
DATA["15"] ="eee"
DATA["17"] ="fff"
DATA["16"] ="faa"
DATA["18"] ="fbb"
DATA["29"] ="fcc"
DATA["20"]="fcd"

class KeywordSource(object):
    implements(IQuerySource)

    def __init__(self, context):
        self.context = context
        self.vocab = SimpleVocabulary(
            [SimpleVocabulary.createTerm(DATA[x], x, DATA[x])
             for x in DATA]
        )

    def __contains__(self, term):
        return self.vocab.__contains__(term)

    def __iter__(self):
        return self.vocab.__iter__()

    def __len__(self):
        return self.vocab.__len__()

    def getTerm(self, value):
        return self.vocab.getTerm(value)

    def getTermByToken(self, value):
        return self.vocab.getTermByToken(value)

    def search(self, query_string):
        q = query_string.lower()
        return [kw
                for kw in self.vocab._terms
                if q.lower() in kw.value]


class KeywordSourceBinder(object):
    implements(IContextSourceBinder)

    def __call__(self, context):
        return KeywordSource(context)


class ITestForm(Interface):

    single_keyword = schema.Choice(
        title=u"Single",
        source=KeywordSourceBinder(),
        required=False)

    keywords = schema.List(
        title=u"Multiple",
        value_type=schema.Choice(
            title=u"Multiple",
            source=KeywordSourceBinder()),
        required=False)

    asingle_keyword = schema.Choice(
        title=u"Ajax Single",
        source=KeywordSourceBinder(),
        required=False)

    akeywords = schema.List(
        title=u"Ajax Multiple",
        value_type=schema.Choice(
            title=u"Multiple",
            source=KeywordSourceBinder()),
        required=False)


class TestForm(form.Form):
    fields = field.Fields(ITestForm)
    fields['single_keyword'].widgetFactory = ChosenFieldWidget
    fields['keywords'].widgetFactory = ChosenMultiFieldWidget
    fields['asingle_keyword'].widgetFactory = AjaxChosenFieldWidget
    fields['akeywords'].widgetFactory = AjaxChosenMultiFieldWidget

    @button.buttonAndHandler(u'Ok')
    def handle_ok(self, action):
        data, errors = self.extractData()
        print data, errors


class ContextLessTestForm(TestForm):
    ignoreContext=True


class TestAdapter(object):
    implements(ITestForm)
    adapts(Interface)

    def __init__(self, context):
        self.context = context

    def _get_single_keyword(self):
        return DATA["11"]

    def _set_single_keyword(self, value):
        print "setting", value

    single_keyword = property(_get_single_keyword, _set_single_keyword)

    def _get_keywords(self):
        return [DATA["11"]]

    def _set_keywords(self, value):
        print "setting", value

    keywords = property(_get_keywords, _set_keywords)

    def _get_single_akeyword(self):
        return DATA["11"]

    def _set_single_akeyword(self, value):
        print "setting", value

    asingle_keyword = property(_get_single_akeyword, _set_single_akeyword)

    def _get_akeywords(self):
        return [DATA["11"]]

    def _set_akeywords(self, value):
        print "setting", value

    akeywords = property(_get_akeywords, _set_akeywords)


TestView = layout.wrap_form(TestForm)
ContextLessTestView = layout.wrap_form(ContextLessTestForm)




"""
Launching all doctests in the tests directory using:

    - the base layer in testing.py

"""

from collective.z3cform.chosen.tests.base import FunctionalTestCase

################################################################################
# GLOBALS avalaible in doctests
# IMPORT/DEFINE objects there or inside ./user_globals.py (better)
# globals from the testing product are also available.
################################################################################
# example:
# from for import bar
# and in your doctests, you can do:
# >>> bar.something
from collective.z3cform.chosen.tests.globals import *
from collective.z3cform.chosen.testing import COLLECTIVE_Z3CFORM_CHOSEN_FUNCTIONAL_TESTING as FUNCTIONAL_TESTING
################################################################################


import unittest2 as unittest
import glob
import os
import logging
import doctest
from plone.testing import layered

optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)

def test_suite():
    """."""
    logger = logging.getLogger('collective.z3cform.chosen.tests')
    cwd = os.path.dirname(__file__)
    pcwd = os.path.dirname(
        os.path.dirname(__file__)
    )
    files = []
    try:
        files = glob.glob(os.path.join(cwd, '*txt'))
        files.extend(glob.glob(os.path.join(pcwd, '*txt')))
        files = [f for f in files if not 'version.txt' in f]
    except Exception,e:
        logger.warn('No doctests for collective.z3cform.chosen')
    suite = unittest.TestSuite()
    globs = globals()
    for s in files:
        suite.addTests([
            layered(
                doctest.DocFileSuite(
                    s, 
                    globs = globs,
                    module_relative=False,
                    optionflags=optionflags,         
                ),
                layer=FUNCTIONAL_TESTING
            ),
        ])
    return suite
    


# vim:set ft=python:



from Testing import ZopeTestCase as ztc
import transaction
from OFS.Folder import Folder

import unittest2 as unittest

from zope.configuration import xmlconfig

from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting as BIntegrationTesting, FunctionalTesting as BFunctionalTesting
from plone.app.testing import TEST_USER_NAME, TEST_USER_ID
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing.selenium_layers import SELENIUM_FUNCTIONAL_TESTING as SELENIUM_TESTING
from plone.testing import zodb, zca, z2

TESTED_PRODUCTS = (\
)

from plone.app.testing import (
    TEST_USER_ROLES,
    TEST_USER_NAME,
    TEST_USER_ID,
    SITE_OWNER_NAME,
)
from plone.app.testing.helpers import (
    login,
    logout,
)

PLONE_MANAGER_NAME = 'Plone_manager'
PLONE_MANAGER_ID = 'plonemanager'
PLONE_MANAGER_PASSWORD = 'plonemanager'

def print_contents(browser, dest='~/.browser.html'):
    """Print the browser contents somewhere for you to see its context
    in doctest pdb, type print_contents(browser) and that's it, open firefox
    with file://~/browser.html."""
    import os
    open(os.path.expanduser(dest), 'w').write(browser.contents)

class Browser(z2.Browser):
    def print_contents(browser, dest='~/.browser.html'):
        return print_contents(browser, dest)

class CollectiveZ3cformChosenLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )
    """Layer to setup the chosen site"""
    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def setUpZope(self, app, configurationContext):
        """Set up the additional products required for the collective.z3cform) site chosen.
        until the setup of the Plone site testing layer.
        """
        self.app = app
        self.browser = Browser(app)
        # old zope2 style products
        for product in TESTED_PRODUCTS:
            z2.installProduct(product)

        # ----------------------------------------------------------------------
        # Import all our python modules required by our packages
        # ---------------------------------------------------------------------
        #with_ploneproduct_jschosen
        import collective.js.chosen
        self.loadZCML('configure.zcml', package=collective.js.chosen)

        # -----------------------------------------------------------------------
        # Load our own chosen
        # -----------------------------------------------------------------------
        import collective.z3cform.chosen
        self.loadZCML('configure.zcml', package=collective.z3cform.chosen)

        # ------------------------------------------------------------------------
        # - Load the python packages that are registered as Zope2 Products
        #   which can't happen until we have loaded the package ZCML.
        # ------------------------------------------------------------------------

        #with_ploneproduct_jschosen
        z2.installProduct(app, 'collective.js.chosen')
        z2.installProduct(app, 'collective.z3cform.chosen')

        # -------------------------------------------------------------------------
        # support for sessions without invalidreferences if using zeo temp storage
        # -------------------------------------------------------------------------
        app.REQUEST['SESSION'] = self.Session()
        if not hasattr(app, 'temp_folder'):
            tf = Folder('temp_folder')
            app._setObject('temp_folder', tf)
            transaction.commit()
        ztc.utils.setupCoreSessions(app)

    def setUpPloneSite(self, portal):
        self.portal = portal
        applyProfile(portal, 'collective.z3cform.chosen:default')


class LayerMixin(object):
    defaultBases = (CollectiveZ3cformChosenLayer() ,)

    def testSetUp(self):
        self.add_user(
            self['portal'],
            PLONE_MANAGER_ID,
            PLONE_MANAGER_NAME,
            PLONE_MANAGER_PASSWORD,
            ['Menager']+TEST_USER_ROLES)

    def add_user(self, portal, id, username, password, roles=None):
        if not roles: roles = TEST_USER_ROLES[:]
        self.loginAsPortalOwner()
        pas = portal['acl_users']
        pas.source_users.addUser(id, username, password)
        setRoles(portal, id, roles)
        self.logout()

    def loginAsPortalOwner(self):
        z2.login(self['app']['acl_users'], SITE_OWNER_NAME)

    def logout(self):
        logout()

class IntegrationTesting(LayerMixin, BIntegrationTesting):
    def testSetUp(self):
        BIntegrationTesting.testSetUp(self)
        LayerMixin.testSetUp(self)

class FunctionalTesting(LayerMixin, BFunctionalTesting):
    def testSetUp(self):
        BFunctionalTesting.testSetUp(self)
        LayerMixin.testSetUp(self)

COLLECTIVE_Z3CFORM_CHOSEN_FIXTURE             = CollectiveZ3cformChosenLayer()
COLLECTIVE_Z3CFORM_CHOSEN_INTEGRATION_TESTING = IntegrationTesting(name = "CollectiveZ3cformChosen:Integration")
COLLECTIVE_Z3CFORM_CHOSEN_FUNCTIONAL_TESTING  = FunctionalTesting( name = "CollectiveZ3cformChosen:Functional")
COLLECTIVE_Z3CFORM_CHOSEN_SELENIUM_TESTING    = FunctionalTesting(bases = (SELENIUM_TESTING, COLLECTIVE_Z3CFORM_CHOSEN_FUNCTIONAL_TESTING,), name = "CollectiveZ3cformChosen:Selenium")

# vim:set ft=python:



import unittest2 as unittest

from collective.z3cform.chosen.testing import (
    COLLECTIVE_Z3CFORM_CHOSEN_FIXTURE as UNIT_TESTING,
    COLLECTIVE_Z3CFORM_CHOSEN_INTEGRATION_TESTING as INTEGRATION_TESTING,
    COLLECTIVE_Z3CFORM_CHOSEN_FUNCTIONAL_TESTING as FUNCTIONAL_TESTING,
    COLLECTIVE_Z3CFORM_CHOSEN_SELENIUM_TESTING as SELENIUM_TESTING,
)
from collective.z3cform.chosen.tests.globals import *

################################################################################
"""


Lot of files generated by the collective.generic packages  will try to load user defined objects in user specific files.
The final goal is to regenerate easyly the test infrastructure on templates updates without impacting
user-specific test boilerplate.
We do not use paster local commands (insert/update) as it cannot determine witch is specific or not and we prefer to totally
separe generated stuff and what is user specific


"""
################################################################################


class TestCase(unittest.TestCase):
    """We use this base class for all the tests in this package.
    If necessary, we can put common utility or setup code in here.
    """
    layer = UNIT_TESTING

    def setUp(self):
        super(TestCase, self).setUp()
        self.portal = self.layer['portal']
        self.app = self.layer['app']

    def add_user(self, id, username, password, roles=None):
        self.layer.add_user( id, username, password, roles=None)

    def logout(self):
        self.layer.logout()

    def login(self, user=None):
        if not user: user = TEST_USER_NAME
        login(self.portal, user)

    def loginAsPortalOwner(self):
        self.layer.loginAsPortalOwner()

    def loginAsManager(self):
        self.login(PLONE_MANAGER_NAME)

class IntegrationTestCase(TestCase):
    """Integration base TestCase."""
    layer = INTEGRATION_TESTING


class FunctionalTestCase(TestCase):
    """Functionnal base TestCase."""
    layer = FUNCTIONAL_TESTING


class SeleniumTestCase(TestCase):
    """Functionnal base TestCase."""
    layer = SELENIUM_TESTING

# vim:set ft=python:



import logging
import transaction
from Products.CMFCore.utils import getToolByName

from collective.z3cform.chosen import app_config
from collective.z3cform.chosen.app_config import PRODUCT_DEPENDENCIES, EXTENSION_PROFILES


def setupVarious(context):
    """Miscellanous steps import handle.
    """
    logger = logging.getLogger('collective.z3cform.chosen / setuphandler')

    # Ordinarily, GenericSetup handlers check for the existence of XML files.
    # Here, we are not parsing an XML file, but we use this text file as a
    # flag to check that we actually meant for this import step to be run.
    # The file is found in profiles/default.

    if context.readDataFile('collective.z3cform.chosen_various.txt') is None:
        return

    portal = context.getSite()

def setupQi(context):
    """Miscellanous steps import handle.
    """
    logger = logging.getLogger('collective.z3cform.chosen / setuphandler')

    # Ordinarily, GenericSetup handlers check for the existence of XML files.
    # Here, we are not parsing an XML file, but we use this text file as a
    # flag to check that we actually meant for this import step to be run.
    # The file is found in profiles/default.

    if context.readDataFile('collective.z3cform.chosen_qi.txt') is None:
        return

    portal = context.getSite() 
    portal_quickinstaller = getToolByName(portal, 'portal_quickinstaller')
    portal_setup = getToolByName(portal, 'portal_setup')
    logger = logging.getLogger('collective.z3cform.chosen.Install')

    for product in PRODUCT_DEPENDENCIES:
        logger.info('(RE)Installing %s.' % product)
        if not portal_quickinstaller.isProductInstalled(product):
            portal_quickinstaller.installProduct(product)
            transaction.savepoint()




import demjson
from ordereddict import OrderedDict

import zope.component
from zope.i18n import translate
from zope.interface import implementsOnly, implementer
from zope.schema.interfaces import ISource, IContextSourceBinder
from zope.schema.interfaces import ITitledTokenizedTerm

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Acquisition import Explicit
from Acquisition.interfaces import IAcquirer
from App.class_init import InitializeClass

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

import z3c.form
from z3c.form.browser import select
from z3c.formwidget.query.widget import (QueryTerms, SourceTerms)

from collective.z3cform.chosen import MessageFactory as _
from collective.z3cform.chosen.interfaces import IChosenWidget


demjson.dumps = demjson.encode
demjson.loads = demjson.decode

def jsbool(value):
    if value:
        return 'true'
    else:
        return 'false'


class ChosenAutocompleteSearch(BrowserView):
    def validate_access(self):

        content = self.context.form.context

        # If the object is not wrapped in an acquisition chain
        # we cannot check any permission.
        if not IAcquirer.providedBy(content):
            return

        url = self.request.getURL()
        view_name = url[len(content.absolute_url()):].split('/')[1]

        # May raise Unauthorized

        # If the view is 'edit', then traversal prefers the view and
        # restrictedTraverse prefers the edit() method present on most CMF
        # content. Sigh...
        if not view_name.startswith('@@') and not view_name.startswith('++'):
            view_name = '@@' + view_name

        view_instance = content.restrictedTraverse(view_name)
        sm = getSecurityManager()
        sm.validate(content, content, view_name, view_instance)

    def __call__(self):
        # We want to check that the user was indeed allowed to access the
        # form for this widget. We can only this now, since security isn't
        # applied yet during traversal.
        self.validate_access()

        query = self.request.get('term', None)
        if not query:
            return ''

        # Update the widget before accessing the source.
        # The source was only bound without security applied
        # during traversal before.
        self.context.update()
        source = self.context.bound_source
        if query:
            terms = set(source.search(query))
        else:
            terms = set()
        self.request.response.setHeader(
            'Content-Type', 'application/javascript')
        return demjson.dumps(
            [(t.token, t.title or t.value or t.token)
             for t in sorted(
                 terms,
                 key=lambda t: t.value or t.title)])


class ChosenBase(select.SelectWidget, Explicit):
    implementsOnly(IChosenWidget)
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    _bound_source = None
    ignoreMissing = True

    input_template = ViewPageTemplateFile('input.pt')
    display_template = ViewPageTemplateFile('display.pt')
    hidden_template = ViewPageTemplateFile('hidden.pt')

    # load values from bounded source at initialisation
    populate_select = False

    # Options passed to jQuery auto-completer
    onselect = ""
    readonly = False
    prompt = True
    promptMessage = _('Select a value')
    noValueMessage = _('no value')
    noValueToken = u'(nothing)'
    no_results_text = _("No results found")
    style = ""
    multiple = False
    allow_single_deselect = False
    search_url = "%s/++widget++%s/@@chosen-autocomplete-search"

    # JavaScript template
    js_template = """\
    (function($) {
        $().ready(function() {
            $('#%(id)s-select').data('klass','%(klass)s').data('title','%(title)s');
            $('#%(id)s-select').chosen({
                allow_single_deselect: %(allow_single_deselect)s,
                no_results_text: '%(no_results_text)s',
                width: '%(width)s'
            });
            %(js_extra)s
            $('#formfield-%(id)s').find('.chzn-container, .chzn-results, .chzn-drop').css({'min-width': '180px'});
        });
    })(jQuery);
    """
    ajax_js_template = """\
    (function($) {
      $().ready(function() {
          $('#%(id)s-select').data('klass','%(klass)s').data('title','%(title)s');
          $('#%(id)s-select').ajaxChosen(
            {
                method: '%(method)s',
                url: '%(url)s',
                dataType: '%(datatype)s',
                minTermLength: 2,
                afterTypeDelay: 0
            },
            %(ajax_callback)s,
            '',
            {allow_single_deselect:%(allow_single_deselect)s,
             no_results_text: '%(no_results_text)s',
             width: '%(width)s'}
          );
          %(js_extra)s
          $('#formfield-%(id)s').find('.chzn-container, .chzn-results, .chzn-drop').css({'min-width': '180px'});
      });
    })(jQuery);
    """
    method = 'GET'
    datatype = 'json'
    ajax_callback = """\
    function (data) {
        var terms = {};
        $.each(data, function (i, val) {
            terms[val[0]] = val[1];
        });
        return terms;
    }"""

    @property
    def source(self):
        """We need to bind the field to the context so that vocabularies
        appear as sources"""
        return self.field.bind(self.context).source

    @property
    def placeholder(self):
        return self.promptMessage

    @property
    def bound_source(self):
        if self._bound_source is None:
            source = self.source
            if IContextSourceBinder.providedBy(source):
                source = source(self.context)
            assert ISource.providedBy(source)
            self._bound_source = source
        return self._bound_source

    @property
    def autocomplete_url(self):
        """Generate the URL that returns autocomplete results for this form
        """
        form_url = self.request.getURL()
        return self.search_url % (form_url, self.name)

    # Override this to insert additional JavaScript
    def js_extra(self):
        return ""

    def __call__(self):
        self.update()
        return self.render()

    def render(self):
        utemplate = None
        if self.mode == z3c.form.interfaces.DISPLAY_MODE:
            utemplate = self.display_template
        if self.mode == z3c.form.interfaces.INPUT_MODE:
            utemplate = self.input_template
        if self.mode == z3c.form.interfaces.HIDDEN_MODE:
            utemplate = self.hidden_template
        if bool(utemplate):
            return utemplate(self)

    def update(self):
        # Allow the source to provide terms until we have more specific ones
        # from the query.
        # Things do not go well if self.terms is None
        self._bound_source = None
        source = self.bound_source
        terms = OrderedDict()

        # populate select values if needed
        if self.populate_select:
            for t in self.source:
                if not t.token in terms:
                    terms[t.token] = t

        # pre defined terms from context+source
        self.terms = SourceTerms(self.context, self.request, self.form, self.field, self, source)

        # If we have values in the request,
        # use these to get the terms.
        request_values = z3c.form.interfaces.NOVALUE
        # extract selected value
        if not self.ignoreRequest:
            request_values = self.extract(
                default=z3c.form.interfaces.NOVALUE)

        if request_values is not z3c.form.interfaces.NOVALUE:
            if not isinstance(request_values, (tuple, set, list)):
                request_values = (request_values,)

            for token in request_values:
                if not token or token == self.noValueToken:
                    continue
                try:
                    t = source.getTermByToken(token)
                    terms[t.token] = t
                except LookupError:
                    # Term no longer available
                    if not self.ignoreMissing:
                        raise

        # take the value from the current saved value
        # if there is an existing adapter allowing it
        if not self.ignoreContext:
            selection = zope.component.getMultiAdapter(
                (self.context, self.field),
                z3c.form.interfaces.IDataManager).query()
            if selection is z3c.form.interfaces.NOVALUE:
                selection = []
            elif not isinstance(selection,
                                (tuple, set, list)):
                selection = [selection]
            for value in selection:
                if not value:
                    continue
                try:
                    t = source.getTerm(value)
                    terms[t.token] = t
                except LookupError:
                    # Term no longer available
                    if not self.ignoreMissing:
                        raise
        # re-set terms with values from request
        self.terms = QueryTerms(self.context, self.request, self.form, self.field, self, terms.values())

        # update widget selected value if any
        select.SelectWidget.update(self)

    def js(self):
        return self.js_template % dict(
            id=self.id,
            url=self.autocomplete_url,
            klass=self.klass,
            allow_single_deselect=jsbool(self.allow_single_deselect),
            title=self.title,
            method=self.method,
            datatype=self.datatype,
            ajax_callback=self.ajax_callback,
            no_results_text=self.no_results_text,
            width=self.width,
            js_extra=self.js_extra())

    @property
    def items(self):
        items = []
        if self.terms is not None:
            # update() has been called
            if ((not self.required or self.prompt)
                and not bool(self.multiple)):
                message = self.noValueMessage
                if self.prompt:
                    message = self.promptMessage
                items.append({
                    'id': self.id + '-novalue',
                    'value': self.noValueToken,
                    'content': message,
                    'selected': self.value == []
                    })
            for count, term in enumerate(self.terms):
                selected = self.isSelected(term)
                item_id = '%s-%i' % (self.id, count)
                content = term.token
                if ITitledTokenizedTerm.providedBy(term):
                    content = translate(
                        term.title,
                        context=self.request,
                        default=term.title)
                items.append(
                    {'id': item_id,
                     'value': term.token,
                     'content': content,
                     'selected': selected})
        return items


class MultiChosenBase(ChosenBase):
    """."""
    multiple = True

    @property
    def source(self):
        return self.field.bind(self.context).value_type.source


class AjaxChosenBase(ChosenBase):
    """."""


class MultiAjaxChosenBase(AjaxChosenBase,
                          MultiChosenBase):
    """."""
    multiple = True

    @property
    def source(self):
        return MultiChosenBase.source.fget(self)


InitializeClass(ChosenBase)
InitializeClass(AjaxChosenBase)


class ChosenSelectionWidget(ChosenBase):
    """widget that allows single selection.
    """
    klass = u'chosen-selection-widget'
    populate_select = True
    width = "280px"


class ChosenMultiSelectionWidget(MultiChosenBase):
    """widget that allows multiple selection
    """
    populate_select = True
    klass = u'chosen-multiselection-widget'
    width = "100%"
    promptMessage = _('Select values')


class AjaxChosenSelectionWidget(AjaxChosenBase):
    """Autocomplete widget that allows single selection.
    """
    klass = u'ajaxchosen-selection-widget'
    js_template = ChosenBase.ajax_js_template
    width="280px"
    promptMessage = _('Enter value')


class AjaxChosenMultiSelectionWidget(MultiAjaxChosenBase):
    """Autocomplete widget that allows multiple selection
    """
    klass = u'ajaxchosen-multiselection-widget'
    js_template = ChosenBase.ajax_js_template
    width="100%"
    promptMessage = _('Enter values')


@implementer(z3c.form.interfaces.IFieldWidget)
def ChosenFieldWidget(field, request):
    return z3c.form.widget.FieldWidget(field,
        ChosenSelectionWidget(request))


@implementer(z3c.form.interfaces.IFieldWidget)
def ChosenMultiFieldWidget(field, request):
    return z3c.form.widget.FieldWidget(field,
        ChosenMultiSelectionWidget(request))


@implementer(z3c.form.interfaces.IFieldWidget)
def AjaxChosenFieldWidget(field, request):
    return z3c.form.widget.FieldWidget(field,
        AjaxChosenSelectionWidget(request))


@implementer(z3c.form.interfaces.IFieldWidget)
def AjaxChosenMultiFieldWidget(field, request):
    return z3c.form.widget.FieldWidget(field,
        AjaxChosenMultiSelectionWidget(request))



#-*- coding: utf-8 -*-
"""Specific project configuration."""
GLOBALS = globals()

################################################################################
# Products that have entries in quickinstaller,
# here are their 'id' (not the translated name)
################################################################################

PRODUCT_DEPENDENCIES = (\
)

EXTENSION_PROFILES = ('collective.z3cform.chosen:default',)

SKIN = 'collective.z3cform.skin'
HIDDEN_PRODUCTS = [u'plone.app.openid', u'NuPlone',
#      u'collective.js.chosen',
#    u'collective.z3cform.chosen.migrations.v1_1',
#    u'collective.z3cform.chosen.migrations',
]
HIDDEN_PROFILES = [u'plone.app.openid', u'NuPlone',
    u'collective.z3cform.chosen.migrations.v11',
    u'collective.z3cform.chosen.migrations',
      u'collective.js.chosen',

]

from zope.interface import implements
from Products.CMFQuickInstallerTool.interfaces import INonInstallable as INonInstallableProducts
from Products.CMFPlone.interfaces import INonInstallable as INonInstallableProfiles


class HiddenProducts(object):
    implements(INonInstallableProducts)

    def getNonInstallableProducts(self):
        return HIDDEN_PRODUCTS


class HiddenProfiles(object):
    implements(INonInstallableProfiles)

    def getNonInstallableProfiles(self):
        return [ u'plone.app.openid', u'NuPlone', ]



import os, sys

from setuptools import setup, find_packages

version = '1.2.2'

def read(*rnames):
    return open(
        os.path.join('.', *rnames)
    ).read()

long_description = "\n\n".join(
    [read('README.rst'),
     read('src', 'collective', 'z3cform', 'chosen', 'README.txt'),
     read('docs', 'INSTALL.rst'),
     read('docs', 'CHANGES.rst'),
    ]
)

classifiers = [
    "Framework :: Plone",
    "Framework :: Plone :: 4.0",
    "Framework :: Plone :: 4.1",
    "Framework :: Plone :: 4.2",
    "Programming Language :: Python",
    "Topic :: Software Development",]

name = 'collective.z3cform.chosen'
setup(
    name=name,
    namespace_packages=[         'collective',         'collective.z3cform',
    ],
    version=version,
    description='chosen widget for z3cform (both chosen & ajax version)',
    long_description=long_description,
    classifiers=classifiers,
    keywords='chosen z3cform widget plone',
    author='kiorky',
    author_email='kiorky@cryptelium.net',
    url='http://pypi.python.org/pypi/%s' % name,
    license='GPL',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data=True,
    install_requires=[
        'setuptools',
        'z3c.autoinclude',
        'Plone',
        'demjson',
        'ordereddict',
        'plone.app.upgrade',
        # with_ploneproduct_jschosen
        'collective.js.chosen >= 1.2',
        'z3c.formwidget.query',
        # -*- Extra requirements: -*-
    ],
    extras_require = {
        'test': ['plone.app.testing',]
    },
    entry_points = {
        'z3c.autoinclude.plugin': ['target = plone',],
    },
)
# vim:set ft=python:



import logging
from zope.i18nmessageid import MessageFactory
MessageFactory = collectivez3cformchosenMessageFactory = MessageFactory('collective.z3cform.chosen') 
logger = logging.getLogger('collective.z3cform.chosen')
def initialize(context):
    """Initializer called when used as a Zope 2 product.""" 

from collective.z3cform.chosen.widget import (
    AjaxChosenFieldWidget,
    AjaxChosenMultiFieldWidget,
    ChosenFieldWidget,
    ChosenMultiFieldWidget,
)
 



# -*- coding: utf-8 -*-

import os, sys

try:
    from Products.CMFPlone.migrations import migration_util
except:
    #plone4
    from plone.app.upgrade import utils as migration_util

from Products.CMFCore.utils import getToolByName


def upgrade(portal_setup):
    """
    """
    site = portal_setup.aq_parent

    # install Products.PloneSurvey and dependencies
    #migration_util.loadMigrationProfile(site,
    #                                    'profile-Products.PloneSurvey:default')
    migration_util.loadMigrationProfile(
        site,
        'profile-collective.z3cform.chosen.migrations.v11:1011')




from zope import interface, schema

import z3c.form

from Products.PluggableAuthService.interfaces.authservice import IPropertiedUser
from plone.theme.interfaces import IDefaultPloneLayer



class IMyPortalUser(IPropertiedUser):
    """ Marker interface implemented by users in my portal. """

class IThemeSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer and a plone skin marker.
    """

class ILayer(interface.Interface):
    """Marker interface that defines a Zope 3 browser layer.
    """

class IChosenWidget(z3c.form.interfaces.ISelectWidget):
    """."""
    onselect = schema.TextLine(
        title=u'On Select',
        description=(u'The ``onselect`` event occurs when a user selects '
                     u'some text in a text field.'),
        required=False)
    readonly = schema.Choice(
        title=u'Read-Only',
        description=(u'When set for a form control, this boolean attribute '
                     u'prohibits changes to the control.'),
        values=(None, 'readonly'),
        required=False)



